import java.util.logging.Logger;
import java.util.ArrayList;
import java.util.Iterator;
import java.io.*;

/** Main class for the warping doors!
 * @author cppchriscpp
 * @version 1.0
 */
public class TravelPortals extends Plugin {
    /**
     * The listener plugin to use.
     */
    static final WarpListener listener = new WarpListener();

    /**
     * A list of all the warp points defined by users.
     */
    public static ArrayList<WarpPoint> warpLocations = new ArrayList<WarpPoint>();

    /*
     * A logging system we can use. (We don't use it much, though...
     */
    private Logger log;

    /**
     * The name of this plugin; used by hMod.
     */
    private String name = "Travel Portals";

    /**
     * The version of this plugin; also used by hMod.
     */
    private String version = "1.0";

    /**
     * Function that is called whenever this plugin is enabled.
     */
    public void enable()
    {
        // Read all the warps from a file.

        try
        {
            FileInputStream fIn = new FileInputStream("doorwarp.ser");
            ObjectInputStream oIn = new ObjectInputStream(fIn);
            warpLocations = (ArrayList<WarpPoint>)oIn.readObject();
            oIn.close();
            fIn.close();
        }
        catch (IOException i)
        {
            // log.warning("Could not load door warps. Starting with 0 warps.");
        }
        catch (java.lang.ClassNotFoundException i)
        {
            // log.severe("ArrayList class not found. What have you done?");
        }

        // Add a little something to /help
        etc.getInstance().addCommand("/portalhelp", "- Shows specific help information for portals.");
    }

    /**
     * Function called when the plugin is disabled or the application is exiting.
     */
    public void disable()
    {
        // save all the warps to a file.
        savedata();

        // remove a little something from /help
        etc.getInstance().removeCommand("/portalhelp");
    }

    public static void savedata()
    {
        try
        {
            FileOutputStream fOut = new FileOutputStream("doorwarp.ser");
            ObjectOutputStream oOut = new ObjectOutputStream(fOut);
            oOut.writeObject(warpLocations);
            oOut.close();
            fOut.close();
        }
        catch (IOException i)
        {
            i.printStackTrace();
            // log.severe("Could not save door warping data!");
        }
    }

    /**
     * Plugin initializer. All of the listeners are enabled here.
     */
    public void initialize()
    {
        log = Logger.getLogger("Minecraft");
        log.info(name + " " + version + " initialized.");
        etc.getLoader().addListener(PluginLoader.Hook.BLOCK_PLACE, listener, this, PluginListener.Priority.MEDIUM);
        etc.getLoader().addListener(PluginLoader.Hook.BLOCK_BROKEN, listener, this, PluginListener.Priority.MEDIUM);
        etc.getLoader().addListener(PluginLoader.Hook.COMMAND, listener, this, PluginListener.Priority.MEDIUM);
        etc.getLoader().addListener(PluginLoader.Hook.PLAYER_MOVE, listener, this, PluginListener.Priority.MEDIUM);
    }

    /**
     * Quick helper function to add warp points.
     * @param w The warp point to add to warpLocations.
     */
    public static void addWarp(WarpPoint w)
    {
        warpLocations.add(w);
    }

}
