import java.util.ArrayList;
import java.util.Iterator;
import java.lang.Math;

/** This is the listener class for the Warp Plugin. Most things actually happen here.
 *
 * @author cppchriscpp
 * @version 1.0
 */
public class WarpListener extends PluginListener {

    /**
     * The numerical type of the block.
     */
    private final int blocktype = 49; // Must be crafted of 49 (obsidian)

    /**
     * A friendly name for the type of block required to construct a portal.
     */
    private final String strBlocktype = "Obsidian";

    /**
     * The numerical type of block used for the doorway. (Don't change this.)
     */
    private final int doortype = 64; // doorway

    /**
     * The type of block used inside the portal.
     * This would be 90 if portal blocks acted like they should.
     */
    private final int portaltype = 9;

    /**
     * Listener function called when a player places a block.
     * @param player The player object that placed the block.
     * @param bp The block that was placed.
     * @param bc The block that was clicked.
     * @param itemInHand The item the user was holding when clicking the block.
     * @return true to block default actions, false otherwise.
     */
    public boolean onBlockPlace(Player player, Block bp, Block bc, Item itemInHand)
    {
        // Did the user place a redstone torch?
        if (player.canUseCommand("/makeportal") && bp.getType() == 76) {

            // Number of walls surrounding the torch
            int numwalls = 0;

            // XZ plane; Y height.
            int x = bp.getX();
            int y = bp.getY();
            int z = bp.getZ();

            // Test the area to the right of the portal
            if (etc.getServer().getBlockAt(x + 1, y, z).getType() == blocktype &&
             etc.getServer().getBlockAt(x + 1, y + 1, z).getType() == blocktype)
                    numwalls++;
            else if (etc.getServer().getBlockAt(x + 1, y, z).getType() == doortype &&
             etc.getServer().getBlockAt(x + 1, y + 1, z).getType() == doortype)
                    numwalls += 10;

            // Test the area to the left of the portal
            if (etc.getServer().getBlockAt(x - 1, y, z).getType() == blocktype &&
             etc.getServer().getBlockAt(x - 1, y + 1, z).getType() == blocktype)
                    numwalls++;
            else if (etc.getServer().getBlockAt(x - 1, y, z).getType() == doortype &&
             etc.getServer().getBlockAt(x - 1, y + 1, z).getType() == doortype)
                    numwalls += 10;

            // Test the area in front of the portal
            if (etc.getServer().getBlockAt(x, y, z + 1).getType() == blocktype &&
             etc.getServer().getBlockAt(x, y + 1, z + 1).getType() == blocktype)
                    numwalls++;
            else if (etc.getServer().getBlockAt(x, y, z + 1).getType() == doortype &&
             etc.getServer().getBlockAt(x, y + 1, z + 1).getType() == doortype)
                    numwalls += 10;

            // Test the area behind the portal
            if (etc.getServer().getBlockAt(x, y, z - 1).getType() == blocktype &&
             etc.getServer().getBlockAt(x, y + 1, z - 1).getType() == blocktype)
                    numwalls++;
            else if (etc.getServer().getBlockAt(x, y, z - 1).getType() == doortype &&
             etc.getServer().getBlockAt(x, y + 1, z - 1).getType() == doortype)
                    numwalls += 10;

            // numwalls will be exactly 13 if there is one door, alongside 3 walls.
            // This is what we want. (x, y-1, z) is the coordinate above the torch, and needs to be empty.
            if (numwalls == 13 && etc.getServer().getBlockAt(x, y + 1, z).getType() == 0)
            {
                // Stick portal blocks in the center of the portal.
                etc.getServer().setBlockAt(portaltype, x, y, z);
                etc.getServer().setBlockData(x, y, z, 0);
                etc.getServer().setBlockAt(portaltype, x, y+1, z);
                etc.getServer().setBlockData(x, y+1, z, 0);

                // Tell the user s/he made a portal
                player.sendMessage("4You have created a portal! Type /portalhelp for help using it.");

                // Actually add it to the warp list.
                TravelPortals.addWarp(new WarpPoint(x,y,z));
                TravelPortals.savedata();

            }

        }
        // Allow the user to place the torch, even though it is immediately replaced with water.
        // (The user is supposed to lose the torch.)
        return false;
    }

    /**
     * Called when a player breaks a block.
     * @param ply The player responsible for the broken block.
     * @param block The block that was broken.
     * @return true to override default behavior, otherwise false.
     */
    public boolean onBlockBreak(Player ply, Block block)
    {
        // Iterate through all warp points and check to make sure the user
        // is nowhere near any of them.
        for (WarpPoint w : TravelPortals.warpLocations)
        {
            // Check if the user actually hit one of them.
            if (((Math.abs(w.getX() - block.getX()) < 2 && Math.abs(w.getZ() - block.getZ()) < 1) || (Math.abs(w.getZ() - block.getZ()) < 2 && Math.abs(w.getX() - block.getX()) < 1)) && (block.getY() - w.getY() < 2 && block.getY() - w.getY() >= 0))
            {
                if (ply.canUseCommand("/makeportal"))
                {
                    // They hit an important warping block..destroy the warp point.
                    // Remove the blocks
                    etc.getServer().setBlockAt(0, w.getX(), w.getY(), w.getZ());
                    etc.getServer().setBlockAt(0, w.getX(), w.getY() + 1, w.getZ());
                    // Remove it from the list of warps
                    TravelPortals.warpLocations.remove(TravelPortals.warpLocations.indexOf(w));
                    TravelPortals.savedata();
                    // Let the user know he's done a bad, bad thing. :<
                    ply.sendMessage("4You just broke a portal.");
                    break;
                }
                else
                {
                    ply.sendMessage("4You cannot break this portal.");
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Run any time a player moves from one block to another.
     * @param player The player that moved.
     * @param from The block the player was over.
     * @param to The block the player is now over.
     */
    public void onPlayerMove(Player player, Location from, Location to)
    {
        if (!player.canUseCommand("/useportal"))
        {
            return;
        }
        // Test to see if the user moved into a portal-type block.
        if (etc.getServer().getBlockAt((int)to.x, (int)to.y, (int)to.z).getType() == portaltype)
        {
            // Yes, they did. Find any nearby warps.
            int w = getWarpFromLocation((int)to.x, (int)to.y, (int)to.z);

            // If we have not found one, abort.
            if (w == -1)
                return;

            // If the portal has no destination, the user can't use it.
            if (!TravelPortals.warpLocations.get(w).hasDestination())
            {
                player.sendMessage("4You need to set this portal's destination first.");
                player.sendMessage("cSee /portalhelp for more information.");
            }
            else
            {
                // Find the portal this one points at.
                int loc = getWarp(TravelPortals.warpLocations.get(w).getDestination());

                // If it is not found, tell the user.
                if (loc == -1)
                {
                    player.sendMessage("4This portal's destination (" + TravelPortals.warpLocations.get(w).getDestination() + ") does not exist.");
                    player.sendMessage("cSee /portalhelp for more information.");
                }
                else // Otherwise teleport the user.
                {
                    int x = TravelPortals.warpLocations.get(loc).getX();
                    int y = TravelPortals.warpLocations.get(loc).getY();
                    int z = TravelPortals.warpLocations.get(loc).getZ();
                    float rotation = 180.0f; // c

                    player.teleportTo((double)x + 0.50, (double)y, (double)z + 0.50, rotation, player.getPitch());
                    // rotation fix

                    if (etc.getServer().getBlockAt(x + 1, y, z).getType() == doortype)
                    {
                        rotation = 270.0f;
                    }
                    else if (etc.getServer().getBlockAt(x, y, z+1).getType() == doortype)
                    {
                        rotation = 0.0f;
                    }
                    else if (etc.getServer().getBlockAt(x - 1, y, z).getType() == doortype)
                    {
                        rotation = 90.0f;
                    }
                    else if (etc.getServer().getBlockAt(x, y, z-1).getType() == doortype)
                    {
                        rotation = 180.0f;
                    }
                    else
                    {
                        // sometimes the rotation is just wrong. I suppose we could store this..
                        // The problem is with the map not preloading the blocks.
                        /*player.sendMessage("Direction-setting failed.");
                        player.sendMessage(etc.getServer().getBlockAt(x + 1, y, z).getType() + " , " + etc.getServer().getBlockAt(x, y, z + 1).getType());
                        player.sendMessage(etc.getServer().getBlockAt(x - 1, y, z).getType() + " , " + etc.getServer().getBlockAt(x, y, z - 1).getType());
                        player.sendMessage(x + "," + y + "," + z);*/
                    }
                    player.setRotation(rotation);
                }
            }
        }
    }

    /**
     * Run whenever a player runs a command.
     * @param player The player running the command.
     * @param split An array of space-delimited words from the command.
     * @return true if the command is handled, false otherwise.
     */
    public boolean onCommand(Player player, String[] split)
    {
        // Help command
        if (player.canUseCommand("/portalhelp") && split[0].equals("/portalhelp"))
        {
            player.sendMessage("3Portal Help:");
            player.sendMessage("cYou can create portals by surrounding a 2x1 block");
            player.sendMessage("cwith " + strBlocktype + " and a door, then putting a");
            player.sendMessage("credstone torch at the bottom.");
            player.sendMessage("c/sethere [name] sets the name of this portal.");
            player.sendMessage("c/warpto [name] sets the name of the portal this one goes to.");
            player.sendMessage("6To use these, you must be within one block of the portal.");
            player.sendMessage("7To get a list of existing portals, use the command /listportals.");
        }

        // List of portals
        else if (player.canUseCommand("/portalhelp") && split[0].equals("/listportals"))
        {
            String allp = "";
            for (WarpPoint w : TravelPortals.warpLocations)
            {
                if (w.hasName())
                    allp += w.getName() + ", ";
            }
            if (allp.length() > 2)
                allp = allp.substring(0, allp.length() - 2);

            player.sendMessage("Portals: " + allp);
        }

        // Set the name of a nearby portal.
        else if (player.canUseCommand("/sethere") && split[0].equals("/sethere"))
        {
            // The user did include a name, right?
            if (split.length < 2)
            {
                player.sendMessage("4You have to include a name for the location!");
            }
            else if (getWarp(split[1]) != -1) // Test to see if the name is already in use.
            {
                player.sendMessage("4There is already a portal named " + split[1] + ". Please pick another location.");
            }
            else // This *should* be a valid portal
            {
                // Check to make sure the user is actually at a portal.
                int loc = getWarpFromLocation((int)Math.round(player.getLocation().x), (int)player.getLocation().y, (int)Math.round(player.getLocation().z));
                // Not found
                if (loc == -1)
                {
                    player.sendMessage("4No portal found! (You must be within one block of the portal.)");
                }
                else // Rename the portal
                {
                    TravelPortals.warpLocations.get(loc).setName(split[1]);
                    TravelPortals.savedata();
                    player.sendMessage("cThis portal is now known as " + split[1] + ".");
                }
            }
        }
        else if (player.canUseCommand("/warpto") && split[0].equals("/warpto"))
        {
            // Check for a nearby portal
            int loc = getWarpFromLocation((int)Math.round(player.getLocation().x), (int)player.getLocation().y, (int)Math.round(player.getLocation().z));
            if (split.length < 2) // We do have a name, correct?
            {
                player.sendMessage("4You have to include a destination!");
            }
            else if(loc == -1) // The user wasn't close enough to a portal.
            {
                player.sendMessage("4No portal found! (You must be within one block of the portal.)");
            }
            else // Set the portal's destination!
            {
                TravelPortals.warpLocations.get(loc).setDestination(split[1]);
                TravelPortals.savedata();
                player.sendMessage("cThis portal now points to " + split[1] + ".");
            }
        }
        else
            return false;
        return true;
    }

    /**
     * Get the index of a warp object from its name.
     * @param name The name of the portal to find.
     * @return The index of the portal in TravelPortals.warpLocations, or -1 if it is not found.
     */
    public int getWarp(String name)
    {
        // Test all warps to see if they have the name we're looking for.
        for (int i = 0; i < TravelPortals.warpLocations.size(); i++)
        {
            if (TravelPortals.warpLocations.get(i).getName().equals(name))
                return i;
        }
        return -1;
    }

    /**
     * Find a warp point from a relative location. (Within 1 block)
     * @param x X coordinate to search near.
     * @param y Y coordinate to search near.
     * @param z Z coordinate to search near.
     * @return The index of a nearby portal in TravelPortals.warpLocations, or -1 if it is not found.
     */
    public int getWarpFromLocation(int x, int y, int z)
    {
        // Iterate through all warps and check how close they are
        for (int i = 0; i < TravelPortals.warpLocations.size(); i++)
        {
            WarpPoint wd = TravelPortals.warpLocations.get(i);
            if (wd.getY() == y)
            {
                // We found one!!
                if (Math.abs(wd.getX() - x) <= 1 && Math.abs(wd.getZ() - z) <= 1)
                    return i;
            }
        }
        return -1;
    }

}
